use leptos::*;
use std::sync::Arc;
use leptos::logging::log;

#[component]
pub fn EditableCell(
    value: String,
    on_input: impl Fn(String) + 'static + Clone,
    key: Arc<String>,
    focused_cell: ReadSignal<Option<String>>,
    set_focused_cell: WriteSignal<Option<String>>,
    on_focus: Option<Callback<()>>,
    on_blur: Option<Callback<()>>,
    input_type: InputType,
    suggestions: Vec<String>,
    show_suggestions: bool,
    on_select_suggestion: Option<Callback<String>>,
) -> impl IntoView {
    let input_ref = create_node_ref::<html::Input>();
    let textarea_ref = create_node_ref::<html::Textarea>();
    let (local_value, set_local_value) = create_signal(value.clone());
    let input_type_clone = input_type.clone();

    // Handle input event with debouncing
    let handle_input = {
        let on_input_clone = on_input.clone(); // Clone the callback
        move |e: web_sys::Event| {
            let new_value = match input_type_clone {
                InputType::Text => event_target_value(&e),
                InputType::TextArea => event_target_value(&e),
            };
            log!("Input event: {}", new_value);
            set_local_value.set(new_value.clone());
            on_input_clone(new_value); // Use the cloned callback
        }
    };

    // Focus handling
    let handle_focus = {
        let key = Arc::clone(&key);
        move |_| {
            log!("Focus gained for key: {}", key);
            set_focused_cell.set(Some(key.to_string()));
            if let Some(on_focus) = &on_focus {
                on_focus.call(());
            }
        }
    };

    // Blur handling
    let handle_blur = {
        let key = Arc::clone(&key);
        move |_| {
            log!("Focus lost for key: {}", key);
            set_focused_cell.set(None);
            if let Some(on_blur) = &on_blur {
                on_blur.call(());
            }
        }
    };

    // Update input field value when focused cell changes
    create_effect(move |_| {
        if focused_cell.get().as_deref() == Some(key.as_str()) {
            log!("Setting focus for key: {}", key);
            if let Some(input) = input_ref.get() {
                let _ = input.focus();
            }
        }
    });

    view! {
        <div class="editable-cell">
            {match input_type {
                InputType::Text => view! {
                    <input
                        type="text"
                        prop:value=move || local_value.get()
                        on:input=handle_input
                        on:focus=handle_focus
                        on:blur=handle_blur
                        node_ref=input_ref
                        class="editable-cell-input"
                    />
                }.into_view(),
                InputType::TextArea => view! {
                    <textarea
                        prop:value=move || local_value.get()
                        on:input=handle_input
                        on:focus=handle_focus
                        on:blur=handle_blur
                        node_ref=textarea_ref
                        class="editable-cell-input"
                    />
                }.into_view()
            }}

            // Render suggestions list
            {move || {
                if show_suggestions && !suggestions.is_empty() {
                    view! {
                        <ul class="editable-cell-suggestions">
                            {suggestions.iter().map(|suggestion| {
                                let suggestion_clone = suggestion.clone();
                                view! {
                                    <li
                                        class="editable-cell-suggestion"
                                        on:click=move |_| {
                                            if let Some(on_select_suggestion) = &on_select_suggestion {
                                                on_select_suggestion.call(suggestion_clone.clone());
                                            }
                                        }
                                    >
                                        {suggestion}
                                    </li>
                                }
                            }).collect::<Vec<_>>()}
                        </ul>
                    }
                } else {
                    view! { <ul></ul> }
                }
            }}
        </div>
    }
}

#[derive(Debug, PartialEq, Eq, Clone)]
pub enum InputType {
    Text,
    TextArea,
}